

%=================================================================================
% This program does the proof of the connecting orbit for the delay henon map.
% The program has a lone run time but a large part of this is due to the fact
%that the program finds and refines the connecting orbit before the proof starts.
%The refining of the connecting orbit is kind of ``hacky'' and could be 
%substantially improved.  However this is not the point of the present work.

%The only know that the user should adjust in this program is the variable 
%``numDim'' at line number 25.
%The proof will run successfully for any choice between 3 and 20.  For values 
%higher than 20 the proof may fail depending on the memory resources of the 
%computer you are using.
%=================================================================================



clear
format long



%startintlab
numDim = 5       %User specified variable.


a = intval('1.6')
b = intval('0.1')

sigma_s = 0.8;
sigma_u = 0.25;



numIterates = 3
numPoints = 2000


s = intval(10^(-3));
epsilon_tol = intval(10^(-15));

N = 25

%==========================================================================
%  Linear Data Computation and Variable Assignment   
%==========================================================================




[p_p, Dfp_p, lambda_p_u, Lambda, xi_p_u, Q, Q_inv, conjVectData] = ...
delayHenon_linearData_p(a, b, numDim);


%Stable eigenspace:                 
A = Q(:, 2:numDim);


%l_s and m_s count
l_s = 0;
m_s = 0;
k = 1;


while k <= numDim-1
    thisConj = conjVectData(k);
    if thisConj == 1
       l_s = l_s + 1; 
       k = k+1;
    else
      m_s = m_s + 1;
      k = k+2;
    end
end



%Plot Eigenvalues:
figure
hold on
plot(mid(real(lambda_p_u)), mid(imag(lambda_p_u)), 'bo')
for k = 1:numDim-1
    plot(mid(real(Lambda(k,k))), imag(mid(Lambda(k,k))), 'ro')
end

Radius = 1;
[circ_1, circ_2, circ_3] = cylinder(Radius,200);
plot(circ_1(1,:), circ_2(1,:), 'g')
plot(0, 0, 'k*')







%======== Validation of the low order (linear) approximation ============== 
%======= of the high dimensional (co-dimension one) stable manifold =======



A = 0.001*A;


[delta_s, nu_s, C_tilde, C_n, K_n, goodComputation] = ...
    validate_coDimOne_stable_henonLike(...
      Lambda, A, Q, Q_inv, s, epsilon_tol, a, b, numDim);
  
  
if goodComputation == 0
   crashNow = crashHere_badStableManifold 
end
  
  
delta_s
nu_s
A_norm = norm(A, inf)  




%Obtain Validated bounds on the derivative of the truncation error.

partialBounds = intval(zeros(numDim-1, 1));
D2_bounds = intval(zeros(numDim-1, 1));

tic

for i = 1:numDim-1
    [delta_s_i, B_i_bound, goodPartial] = ...
        validate_truncationErrorPartial_henonLike(...
            i, Lambda, lambda_p_u, A, Q, Q_inv, ...
            nu_s, delta_s, C_tilde, a,b, numDim);

    if goodPartial == 0
       crashNow = crashHere; 
    end

    
    partialBounds(i) = 2*B_i_bound*nu_s + delta_s_i;
    
    
    D2_bounds(i) = 2*B_i_bound + ... 
    (2*intval('pi')*(l_s + 4*m_s)/(nu_s*sigma_s))*delta_s_i;


end


partialBounds

D2_bounds

Dh_bound = partialBounds'*ones(numDim-1, 1)
D2h_bound = (numDim-1)*D2_bounds'*ones(numDim-1, 1)





%=============== One Dim Parameterization (High Order) ====================


scale = intval('1')

[Pu] = oneDimManifold_intval_delayHenon(a, b, ...
    numDim, N, p_p, lambda_p_u, xi_p_u, scale);


%=============== Validate the High Order ==================================
%========= Parameterization of the one dim manifold ====================== 

N_f = 1;
N_dim = numDim;
nu_u = intval(0.8);


s_u = 10^(-1);


[delta_u, delta_max_u, Pu_norm, delta_u_good, orderTest_u] = ...
    validateUnstableManifold_1D_delayHenon(...
        Pu, p_p, lambda_p_u, xi_p_u, ...
        N_f, N_dim, nu_u, ...
        s_u, N, a, b, numDim);


inf(nu_u)    
sup(delta_u)
inf(delta_max_u)


sup(delta_u)
sup(delta_s)



%===============  Find Rough Guess ========================================



[theta_n, xn, tau_n, thisError] = ...
        findGuess_prodSpaceHighDim(...
        numIterates, numPoints, ...
        p_p, lambda_p_u, Lambda, ...
        A, conjVectData, Pu, nu_u, sigma_u, ...
        N, a, b, numDim)


tau_cylinderNorm = polyCylinderNorm(tau_n, conjVectData)



%=========Extend the ``time of flight'' for next Newton====================




theta_n1 = theta_n
xn1 = xn
tau_n1 = tau_n


k = 0;
k_max = 1000;

while (k <= k_max) & (tau_cylinderNorm >= 0.5*exp(-sigma_s)*inf(nu_s))

k = k+1;
    
Ps_tau = evaluate_linearParm(...
    mid(p_p), mid(A), tau_n1, conjVectData, numDim);  

xn1 = [xn1; Ps_tau];

tau_n1 = tauLinearDynamics_real(tau_n1, mid(Lambda), conjVectData, numDim);
%tau_n1 = mid(Lambda*tau_n1);


[theta_n1, xn1, tau_n1, error_Newton] = ...
    homoclinicNewton_prod_delayHenon(...
    theta_n1, xn1, tau_n1, numIterates-1, 1+k, mid(Pu), N, ...
    mid(p_p), mid(A), mid(a), mid(b), conjVectData, numDim);


goal = 0.5*exp(-sigma_s)*inf(nu_s)
tau_cylinderNorm = polyCylinderNorm(tau_n1, conjVectData)

n_u = numIterates - 1
n_s = 1+k

end




figure
hold on
numPoints = floor(size(xn1, 1)/numDim);



for k=1:numPoints
   orbit(k) = xn1(numDim*(k-1) + 1);
end

plot(orbit, 'g')


%figure 
%hold on
%plot(orbit(1:25), 'g')


%stopNow = stopHere


figure
hold on

orbitBack = zeros(1, 25);
parmOrbit = zeros(1, 25);
parmPoint = theta_n1
for k = 1:25
    parmOrbit(25 - k + 1) = mid(parmPoint);
    parmPoint = mid((1/lambda_p_u)*parmPoint);
end


for k = 1:25
   
    Pu_theta = intval(zeros(numDim, 1));
    for j = 1:numDim
        Pu_theta(j) = horner(parmOrbit(k), Pu(j, :), N); 
    end
    orbitBack(k) =  mid(Pu_theta(1));
end


orbit = [orbitBack, orbit]

figure 
hold on
orbitSize = size(orbit, 2);
orbitAxis = linspace(0, orbitSize-1, orbitSize)
plot(orbitAxis(1:25), orbitBack,  'r')
plot(orbitAxis(25:orbitSize), orbit(25:orbitSize), 'b')
plot(orbitAxis(25), orbit(25), 'k*')
plot(orbitAxis(orbitSize), orbit(orbitSize), 'k*')
%plot(orbitAxis(26:orbitSize), orbit(26:orbitSize), 'go')




%==============  Validation of the Approximate Connection   ===============

tic


theta_n1_int = intval(theta_n1);
xn1_int = intval(xn1);
tau_n1_int = intval(tau_n1);




if sup(abs(theta_n1_int)) >= nu_u
   crashNow = theta_bad 
end
if tau_cylinderNorm >= nu_s
   crashNow = tau_bad 
end



F_N = F_prod_delayHenon_intval(...
    theta_n1_int, xn1_int, tau_n1_int, n_u, n_s, ...
    p_p, A, Pu, N, a, b, conjVectData, numDim)';


NK_prodDim = size(F_N, 1)



epsilon_x = norm(F_N, inf);
errorBound_float = sup(epsilon_x)


DF_N = DF_prod_delayHenon_intval(...
    theta_n1_int, xn1_int, tau_n1_int, n_u, n_s, ...
    p_p, A, Pu, N, a, b, conjVectData, numDim);



Id = intval(eye(NK_prodDim));
A_approx = intval(inv(mid(DF_N)));
A_approxNorm = norm(A_approx, inf)



delta_A = norm(Id - A_approx*DF_N, inf)
epsilon_NK_1 = A_approxNorm*epsilon_x
epsilon_NK = norm(A_approx*F_N)


%Include the Trunctation Errors:

the_K = size(A_approx, 1)/numDim - 1
A_approx_s =  A_approx(:, numDim*the_K + 1:size(A_approx, 1));
A_approx_u = A_approx(:, 1:numDim);


A_norm_s = norm(A_approx_s, inf);
A_norm_u = norm(A_approx_u, inf);


epsilon_hat = epsilon_NK + A_norm_s*delta_s + A_norm_u*delta_u



sigmaHat_u = -log(abs(theta_n1_int)/nu_u);
tau_cylinderNorm = polyCylinderNorm(tau_n1_int, conjVectData)
sigmaHat_s = -log(tau_cylinderNorm/nu_s);



r_u = exp(-sigma_u)*nu_u - abs(theta_n1_int)
r_s = exp(-sigma_s)*nu_s - tau_cylinderNorm



if sup(r_u) < inf(r_s)
   r_kappa = r_u; 
end



if sup(sigmaHat_u) >= 1
    sigmaHat_u = intval(1);
end


if sup(sigmaHat_s) >= 1
   sigmaHat_s = intval(1);
end



delta_hat = delta_A + ...
    ((intval('pi')*2*A_norm_u)/(nu_u*sigmaHat_u))*delta_u + ...
    partialBounds'*ones(numDim-1, 1)




%Lipschitz Constant:

C_f_inv = 2*abs(a)/abs(b)
C_f = 2*abs(a)


r_kappa = r_s

B_theta = midrad(mid(theta_n1_int), inf(r_kappa));
B_tau = midrad(mid(tau_n1_int), inf(r_kappa));


thisSum = intval(0);

for k = 0:N-2
    thisSum = thisSum + (k+2)*(k+1)*norm( Pu(:, k+1), inf)*B_theta;
end

D2_PNu_bound = thisSum + 2*abs(a)/abs(b)
D2_QNs_bound = norm(A, inf) + 2*abs(a)
D2_hu_bound = (intval('pi')^2/((nu_u^2)*(sigma_u^2)))*delta_u
D2_hs_bound = ones(1,numDim-1)*D2_bounds



kappa = D2_QNs_bound + D2_hs_bound

if sup(kappa) < inf(D2_PNu_bound + D2_hu_bound)
    kappa = D2_PNu_bound + D2_hu_bound
    unstableBigger_D2 = 1
end


discrimenant = 4*A_approxNorm*kappa*epsilon_hat
    

r_min = ((1 - delta_hat) - sqrt((1 - delta_hat)^2 - discrimenant))/(2*A_approxNorm*kappa)


if (sup(r_min) > inf(r_kappa))
    crashNow = crashHere
end


if sup(delta_hat + A_approxNorm*kappa*r_min) >= 1
    crashNow = crashHere
else
    goodConnectionProof = 1
end


toc


%Plotting:










