function varargout = homoclinicPoint_highDim(...
    theta_0, phi_0, n_u, n_s, Pu, p, A, N, a, b, dimNum)


%Theta is the unstable variable.
%Phi is the stable one.

xn = [theta_0;
      phi_0];

  
thisError = inf;   
numLoops = 0;   


%Compute the derivative of the parameterization
for j = 2:N+1
    DPu(:, j-1) = (j-1)*Pu(:, j);
end


%NEWTON LOOP:   ===========================================================

while (thisError > 5*10^(-12) & numLoops < 25)
 
  numLoops = numLoops + 1
 %Linear Image of theta_n under Ps:
  Ps_phi = real(p + A*xn(2:dimNum, 1));
 
 
 %Image of z_n = (s_n, t_n) Pu:
 for j = 1:dimNum
    Pu_theta(j) = horner(xn(1), Pu(j, :), N); 
 end
 
 %Differential of Ps(theta_n):
 DPs_phi = A;
 %Differential of Pu(z_n):
 for j = 1:dimNum
 DPu_theta(j) = horner(xn(1), DPu(j,:), N-1);
 end
 
 %Evaluate Phi at x_n:
 Phi_xn =  iterate_point_k_highDim(Pu_theta, n_u, a, b, dimNum) ...
         - inverse_iterate_point_k_highDim(Ps_phi, n_s, a, b, dimNum);
 thisError = norm(Phi_xn, inf)
 
  
 %Evaluate Composition Derivatives: 
 DPhi_theta = Dfk_circ_theta_highDim(n_u, xn(1), Pu, DPu, N, a, b, dimNum);
 
 DPhi_phi = Dfinvk_circ_phi_highDim(n_s, xn(2:dimNum), p, A, a, b, dimNum);
 
 
 %Compute differential of Phi:
 DPhi_xn = [DPhi_theta, -DPhi_phi];
            
 %Compute the Newton Step:
 xnew = xn - inv(DPhi_xn)*Phi_xn;
 xn = xnew;
end
 
theta = xnew(1);
tauBar = xnew(2:dimNum);
error = thisError;

  
varargout(1,1)={theta};
varargout(2,1)={tauBar};
varargout(3,1) = {error};
